import numpy as np
from pyTurbulence.syntheticTurbulence import compute_solenoidal_pressure
from pyTurbulence.poissonSolver import poisson_3d
import warnings

def test_compute_solenoidal_pressure():
    # Define a simple test case
    nx, ny, nz = 5, 5, 5
    hx, hy, hz = 2.0, 2.0, 2.0
    domain_resolution = (nx, ny, nz)
    domain_size = (hx, hy, hz)

    # Create velocity components with known gradients
    u = np.zeros((nx, ny, nz))
    v = np.zeros((nx, ny, nz))
    w = np.zeros((nx, ny, nz))

    x = np.linspace(-1,1, nx)
    y = np.linspace(-1,1, ny)
    z = np.linspace(-1,1, nz)

    for i in range(nx):
        for j in range(ny):
            for k in range(nz):
                u[i, j, k] = np.sin(x[i] * 2.*np.pi)
                v[i, j, k] = np.sin(y[j] * 2.*np.pi)
                w[i, j, k] = np.sin(z[k] * 2.*np.pi)

    # Compute the solenoidal pressure field
    pressure = compute_solenoidal_pressure(u, v, w, domain_resolution, domain_size)

    # Define the expected result
    expected_rhs = np.zeros((nx, ny, nz))
    for i in range(nx):
        for j in range(ny):
            for k in range(nz):
                dudx = 2.*np.pi * np.cos(x[i] * 2.*np.pi)
                dvdy = 2.*np.pi * np.cos(y[j] * 2.*np.pi)
                dwdz = 2.*np.pi * np.cos(z[k] * 2.*np.pi)
                expected_rhs[i, j, k] = -1 * (dudx**2 + dvdy**2 + dwdz**2)

    # Compute the expected pressure using the Poisson solver
    expected_pressure = poisson_3d(expected_rhs, nx, ny, nz, hx/nx, hy/nx, hz/nx)
    # Test the results
    assert np.allclose(pressure, expected_pressure, atol=1e-2), "Test failed: pressure does not match expected results"

test_compute_solenoidal_pressure()